#_______________________________________________________________________________
#
#                         edam's arduino makefile
#_______________________________________________________________________________
#                                                                    version 0.1
#
# Copyright (c) 2011 Tim Marston <tim@ed.am>.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
#_______________________________________________________________________________
#
#
# This is a general purpose makefile for use with Arduino (arduino.cc)
# hardware and software.  It works with the arduino-1.0 release and
# requires that to be downloaded separately.  It can be downloaded
# from http://ed.am/dev/make/arduino-makefile where you can also find
# more information and documentation on it's use.  The following text
# can only really be considered a reference to it's use.
#
# There are two ways to use this file, an automatic mode and a manual
# mode.  In automatic mode, you simply copy this makefile to your
# arduino project directory, rename it "Makefile" and type make.  The
# project directory is expected to contain an .ino or .pde file, which
# will automatically be used automatically, along with any other .c,
# .cc or .cpp files in the project directory and any subdirectory
# named "utility".  In this way, this makefile should act as a drop-in
# replacement for the Arduino IDE's build process and can build a
# project automatically from the files in a project directory.
#
# Alternatively, you can manually specify what files should be
# inclided in a build.  To use this makefile manually, you might be
# better to keep it somewhere and include it in your project's
# Makefile after having defined certain parameters that control the
# build.  As an example, consider the following Makefile:
#
#       TARGET = my_program
#       SOURCES = main.cc foo.cc
#		BOARD = nano
#		SERIALDEV = /dev/ttyACM0
#       include ~/src/arduino.mk
#
# In both manual and automatic modes, the standard Arduino main.cpp's
# main() is included, which expects to be able to call init() and
# loop() in your code.  The main difference is that, in manual mode,
# these would typically be placed in a .cc or .cpp file.
#
# When using manual mode, the following variables can be used:
#
# TARGET       The name of the target file.  This is typically the same name
#              as the project directory for an arduino project and, if
#              unspecified, that is used as a default.
#
# SOURCES      A list of all source files of whatever language. The language
#              type is determined by the file extension.
#
# BOARD        Specify a target board type.  These are defined in boards.txt,
#              which came with your arduino installation.  If unspecified,
#              a default is used.  (See below).
#
# SERIALDEV    The unix device of the device where the arduino can be found.
#              If unspecified, a default is used.  (See below).
#
# This general-purpose makefile also defines the following goals for use on the
# command line when you run make:
#
# all          This is the default if no goal is specified. It builds the
#              target and uploads it.
#
# target       Builds the target of your Makefile.
#
# upload       Uploads the target to an attached arduino.
#
# clean        Deletes temporary files.
#
# <file>       Builds the specified file, either an object file or the target,
#              from those that that would be built for the project.
#
#_______________________________________________________________________________
#                                                                       SETTINGS

# The full path to the arduino software, from arduino.cc
ARDUINODIR := $(wildcard ~/opt/arduino-1.0)

# The board name to build for and upload to. For a complete list of available
# boards, see hardware/arduino/boards.txt in your arduino software directory.
# Here is a list of available boards at time of writing:
#   uno         Arduino Uno
#   atmega328   Arduino Duemilanove w/ ATmega328
#   diecimila   Arduino Diecimila or Duemilanove w/ ATmega168
#   nano328     Arduino Nano w/ ATmega328
#   nano        Arduino Nano w/ ATmega168
#   mega2560    Arduino Mega 2560 or Mega ADK
#   mega        Arduino Mega (ATmega1280)
#   mini328     Arduino Mini w/ ATmega328
#   mini        Arduino Mini w/ ATmega168
#   fio         Arduino Fio
#   bt328       Arduino BT w/ ATmega328
#   bt          Arduino BT w/ ATmega168
#   lilypad328  LilyPad Arduino w/ ATmega328
#   lilypad     LilyPad Arduino w/ ATmega168
#   pro5v328    Arduino Pro or Pro Mini (5V, 16 MHz) w/ ATmega328
#   pro5v       Arduino Pro or Pro Mini (5V, 16 MHz) w/ ATmega168
#   pro328      Arduino Pro or Pro Mini (3.3V, 8 MHz) w/ ATmega328
#   pro         Arduino Pro or Pro Mini (3.3V, 8 MHz) w/ ATmega168
#   atmega168   Arduino NG or older w/ ATmega168
#   atmega8     Arduino NG or older w/ ATmega8
ifndef BOARD
BOARD := nano
endif

# The name of the serial device that the arduino is at. For example,
# /dev/ttyACM0 (Uno), or /dev/ttyUSB0 (Duemilanove)
ifndef SERIALDEV
SERIALDEV := /dev/ttyACM0
endif

#_______________________________________________________________________________
#

# auto mode?
INOFILE := $(wildcard *.ino *.pde)
ifdef INOFILE
ifneq ($(words $(INOFILE)), 1)
$(error There is more than one .pde or .ino file in the directory!)
endif
TARGET := $(basename $(INOFILE))
SOURCES := $(INOFILE) \
	$(wildcard *.c *.cc *.cpp $(addprefix utility/, *.c *.cc *.cpp))
endif

# files
OBJECTS := $(addsuffix .o, $(basename $(SOURCES)))
ifdef INOFILE
OBJECT += main.o
endif
ARDUINOSRCDIR := $(ARDUINODIR)/hardware/arduino/cores/arduino
ARDUINOLIB := _arduino.a
ARDUINOLIBTMP := _arduino.a.tmp
ARDUINOSOURCES := $(wildcard $(ARDUINOSRCDIR)/*.c $(ARDUINOSRCDIR)/*.cpp)
ARDUINOOBJECTS := $(addsuffix .o, $(addprefix $(ARDUINOLIBTMP)/, $(basename \
	$(notdir $(ARDUINOSOURCES)))))

# obtain board parameters from the arduino boards.txt file
BOARDS_FILE := $(ARDUINODIR)/hardware/arduino/boards.txt
BOARD_BUILD_MCU := \
	$(shell sed -ne "s/$(BOARD).build.mcu=\(.*\)/\1/p" $(BOARDS_FILE))
BOARD_BUILD_FCPU := \
	$(shell sed -ne "s/$(BOARD).build.f_cpu=\(.*\)/\1/p" $(BOARDS_FILE))
BOARD_BUILD_VARIANT := \
	$(shell sed -ne "s/$(BOARD).build.variant=\(.*\)/\1/p" $(BOARDS_FILE))
BOARD_UPLOAD_SPEED := \
	$(shell sed -ne "s/$(BOARD).upload.speed=\(.*\)/\1/p" $(BOARDS_FILE))
BOARD_UPLOAD_PROTOCOL := \
	$(shell sed -ne "s/$(BOARD).upload.protocol=\(.*\)/\1/p" $(BOARDS_FILE))

# software
CC := avr-gcc
CXX := avr-g++
LD := avr-ld
AR := avr-ar
OBJCOPY := avr-objcopy
AVRDUDE := avrdude
#$(ARDUINODIR)/hardware/tools/avrdude

# flags
CPPFLAGS = -Os -Wall -fno-exceptions -ffunction-sections -fdata-sections
CPPFLAGS += -mmcu=$(BOARD_BUILD_MCU) -DF_CPU=$(BOARD_BUILD_FCPU)
CPPFLAGS += -I. -Iutility -I$(ARDUINOSRCDIR)
CPPFLAGS += -I$(ARDUINODIR)/hardware/arduino/variants/$(BOARD_BUILD_VARIANT)/
AVRDUDEFLAGS = -C $(ARDUINODIR)/hardware/tools/avrdude.conf -DV
AVRDUDEFLAGS += -p $(BOARD_BUILD_MCU) -P $(SERIALDEV)
AVRDUDEFLAGS += -c $(BOARD_UPLOAD_PROTOCOL) -b $(BOARD_UPLOAD_SPEED)
LINKFLAGS = -Os -Wl,--gc-sections -mmcu=$(BOARD_BUILD_MCU)

# checks
ifeq ($(TARGET),)
$(error No target defined!)
endif

# default rule
.DEFAULT_GOAL := all

#_______________________________________________________________________________
#                                                                          RULES

.PHONY:	all target upload clean

all: target upload

target: $(TARGET).hex

upload:
	stty -F $(SERIALDEV) hupcl
	$(AVRDUDE) $(AVRDUDEFLAGS) -U flash:w:$(TARGET).hex:i

clean:
	rm -f $(OBJECTS)
	rm -f $(TARGET).elf $(TARGET).hex $(ARDUINOLIB) *~
	rm -rf $(ARDUINOLIBTMP)

# building the target

$(TARGET).hex: $(TARGET).elf
	$(OBJCOPY) -O ihex -R .eeprom $< $@

.INTERMEDIATE: $(TARGET).elf

$(TARGET).elf: $(ARDUINOLIB) $(OBJECTS)
	$(CC) $(LINKFLAGS) $(OBJECTS) $(ARDUINOLIB) -o $@

%.o: %.ino
	$(COMPILE.cpp) -o $@ -x c++ -include $(ARDUINOSRCDIR)/Arduino.h $<

%.o: %.pde
	$(COMPILE.cpp) -o $@ -x c++ -include $(ARDUINOSRCDIR)/Arduino.h $<

# building the arduino library

$(ARDUINOLIB): $(ARDUINOOBJECTS)
	$(AR) rcs $@ $?
	rm -rf $(ARDUINOLIBTMP)

.INTERMEDIATE: $(ARDUINOOBJECTS)

$(ARDUINOLIBTMP)/%.o: $(ARDUINOSRCDIR)/%.c
	@test -d $(ARDUINOLIBTMP) || mkdir $(ARDUINOLIBTMP)
	$(COMPILE.c) -o $@ $<

$(ARDUINOLIBTMP)/%.o: $(ARDUINOSRCDIR)/%.cpp
	@test -d $(ARDUINOLIBTMP) || mkdir $(ARDUINOLIBTMP)
	$(COMPILE.cpp) -o $@ $<
